@@ -1,11 +1,11 @@
 use strict;
 use warnings;
-package JSON::Schema::Draft201909; # git description: v0.026-35-gd69148e
+package JSON::Schema::Draft201909; # git description: v0.027-26-g81fb247
 # vim: set ts=8 sts=2 sw=2 tw=100 et :
 # ABSTRACT: Validate data against a schema
 # KEYWORDS: JSON Schema data validation structure specification
 
-our $VERSION = '0.027';
+our $VERSION = '0.028';
 
 use 5.016;  # for fc, unicode_strings features
 no if "$]" >= 5.031009, feature => 'indirect';
@@ -33,6 +33,8 @@ use JSON::Schema::Draft201909::Document;
 use JSON::Schema::Draft201909::Utilities qw(get_type canonical_schema_uri E abort annotate_self);
 use namespace::clean;
 
+our @CARP_NOT = qw(JSON::Schema::Draft201909::Document);
+
 has output_format => (
   is => 'ro',
   isa => Enum(JSON::Schema::Draft201909::Result->OUTPUT_FORMATS),
@@ -84,8 +86,8 @@ has _format_validations => (
 );
 
 sub add_schema {
+  croak 'insufficient arguments' if @_ < 2;
   my $self = shift;
-  die 'insufficient arguments' if @_ < 1;
 
   # TODO: resolve $uri against $self->base_uri
   my $uri = !is_ref($_[0]) ? Mojo::URL->new(shift)
@@ -138,8 +140,9 @@ sub add_schema {
 }
 
 sub evaluate_json_string {
+  croak 'evaluate_json_string called in void context' if not defined wantarray;
+  croak 'insufficient arguments' if @_ < 3;
   my ($self, $json_data, $schema, $config_override) = @_;
-  die 'insufficient arguments' if @_ < 3;
 
   my $data;
   try {
@@ -168,17 +171,17 @@ sub evaluate_json_string {
 # $id and $anchor keywords within.
 # Returns the internal $state object accumulated during the traversal.
 sub traverse {
+  croak 'insufficient arguments' if @_ < 2;
   my ($self, $schema_reference, $config_override) = @_;
-  die 'insufficient arguments' if @_ < 2;
 
-  my $base_uri = Mojo::URL->new($config_override->{canonical_schema_uri} // '');
+  my $base_uri = Mojo::URL->new($config_override->{initial_schema_uri} // '');
 
   my $state = {
     depth => 0,
     data_path => '',                    # this never changes since we don't have an instance yet
-    traversed_schema_path => '',        # the accumulated path up to the last $ref traversal
-    canonical_schema_uri => $base_uri,  # the canonical path of the last traversed $ref
-    schema_path => '',                  # the rest of the path, since the last traversed $ref
+    traversed_schema_path => '',        # the accumulated traversal path up to the last $ref traversal
+    initial_schema_uri => $base_uri,    # the canonical URI as of the start or the last traversed $ref
+    schema_path => '',                  # the rest of the path, since the start or the last traversed $ref
     errors => [],
     # for now, this is hardcoded, but in the future we will wrap this in a dialect that starts off
     # just with the Core vocabulary and then determine the actual vocabularies from the '$schema'
@@ -186,7 +189,6 @@ sub traverse {
     vocabularies => [
       (map use_module('JSON::Schema::Draft201909::Vocabulary::'.$_)->new,
         qw(Core Applicator Validation Format Content MetaData)),
-      $self,  # for discontinued keywords defined in the base schema
     ],
     identifiers => [],
     configs => {},
@@ -211,30 +213,17 @@ sub traverse {
 }
 
 sub evaluate {
+  croak 'evaluate called in void context' if not defined wantarray;
+  croak 'insufficient arguments' if @_ < 3;
   my ($self, $data, $schema_reference, $config_override) = @_;
-  die 'insufficient arguments' if @_ < 3;
 
   my $base_uri = Mojo::URL->new;  # TODO: will be set by a global attribute
 
   my $state = {
-    depth => 0,
     data_path => '',
-    traversed_schema_path => '',        # the accumulated path up to the last $ref traversal
-    canonical_schema_uri => $base_uri,  # the canonical path of the last traversed $ref
-    document => 'SEE BELOW',            # the ::Document object containing this schema
-    document_path => 'SEE BELOW',       # the *initial* path within the document of this schema
-    schema_path => '',                  # the rest of the path, since the last traversed $ref
-    errors => [],
-    annotations => [],
-    seen => {},
-    # for now, this is hardcoded, but in the future the dialect will be determined by the
-    # traverse() pass on the schema and examination of the referenced metaschema.
-    vocabularies => [
-      (map use_module('JSON::Schema::Draft201909::Vocabulary::'.$_)->new,
-        qw(Core Applicator Validation Format Content MetaData)),
-      $self,  # for discontinued keywords defined in the base schema
-    ],
-    evaluator => $self,
+    traversed_schema_path => '',        # the accumulated traversal path up to the last $ref traversal
+    initial_schema_uri => $base_uri,    # the canonical URI as of the start or the last traversed $ref
+    schema_path => '',                  # the rest of the path, since the start or the last traversed $ref
   };
 
   my $valid;
@@ -247,7 +236,7 @@ sub evaluate {
     }
     else {
       # traverse is called via add_schema -> ::Document->new -> ::Document->BUILD
-      $document = $self->add_schema($state->{canonical_schema_uri}, $schema_reference);
+      $document = $self->add_schema($base_uri, $schema_reference);
       ($schema, $canonical_uri) = map $document->$_, qw(schema canonical_uri);
       $document_path = '';
     }
@@ -256,17 +245,30 @@ sub evaluate {
       if not defined $schema;
 
     $state = +{
+      %$state,
+      depth => 0,
+      initial_schema_uri => $canonical_uri,   # the canonical URI as of the start or the last traversed $ref
+      document => $document,                  # the ::Document object containing this schema
+      document_path => $document_path,        # the *initial* path within the document of this schema
+      errors => [],
+      annotations => [],
+      seen => {},
+      # for now, this is hardcoded, but in the future the dialect will be determined by the
+      # traverse() pass on the schema and examination of the referenced metaschema.
+      vocabularies => [
+        (map use_module('JSON::Schema::Draft201909::Vocabulary::'.$_)->new,
+          qw(Core Applicator Validation Format Content MetaData)),
+      ],
+      evaluator => $self,
       %{$document->evaluation_configs},
       (map {
         my $val = $config_override->{$_} // $self->$_;
         defined $val ? ( $_ => $val ) : ()
       } qw(short_circuit collect_annotations validate_formats annotate_unknown_keywords)),
-      %$state,
     };
 
-    @$state{qw(canonical_schema_uri document document_path)} = ($canonical_uri, $document, $document_path);
-
     $valid = $self->_eval($data, $schema, $state);
+    warn 'result is false but there are no errors' if not $valid and not @{$state->{errors}};
   }
   catch ($e) {
     if ($e->$_isa('JSON::Schema::Draft201909::Result')) {
@@ -294,8 +296,8 @@ sub evaluate {
 }
 
 sub get {
+  croak 'insufficient arguments' if @_ < 2;
   my ($self, $uri) = @_;
-  die 'insufficient arguments' if @_ < 2;
 
   my ($subschema, $canonical_uri) = $self->_fetch_schema_from_uri($uri);
   $subschema = dclone($subschema) if is_ref($subschema);
@@ -305,6 +307,7 @@ sub get {
 ######## NO PUBLIC INTERFACES FOLLOW THIS POINT ########
 
 sub _traverse {
+  croak 'insufficient arguments' if @_ < 3;
   my ($self, $schema, $state) = @_;
 
   delete $state->{keyword};
@@ -333,7 +336,15 @@ sub _traverse {
   }
 }
 
+# keyword => undef, or arrayref of alternatives
+my %removed_keywords = (
+  definitions => [ '$defs' ],
+  dependencies => [ qw(dependentSchemas dependentRequired) ],
+);
+
 sub _eval {
+  croak '_eval called in void context' if not defined wantarray;
+  croak 'insufficient arguments' if @_ < 4;
   my ($self, $data, $schema, $state) = @_;
 
   # do not propagate upwards changes to depth, traversed paths,
@@ -376,7 +387,12 @@ sub _eval {
       next if not $vocabulary->can($method);
 
       $state->{keyword} = $keyword;
-      $valid = 0 if not $vocabulary->$method($data, $schema, $state);
+      my $error_count = @{$state->{errors}};
+      if (not $vocabulary->$method($data, $schema, $state)) {
+        warn 'result is false but there are no errors (keyword: '.$keyword.')'
+          if $error_count == @{$state->{errors}};
+        $valid = 0;
+      }
 
       last ALL_KEYWORDS if not $valid and $state->{short_circuit};
 
@@ -384,6 +400,20 @@ sub _eval {
     }
   }
 
+  # check for previously-supported but now removed keywords
+  foreach my $keyword (sort keys %removed_keywords) {
+    next if not exists $schema->{$keyword};
+    my $message ='no-longer-supported "'.$keyword.'" keyword present (at location "'
+      .canonical_schema_uri($state).'")';
+    if ($removed_keywords{$keyword}) {
+      my @list = map '"'.$_.'"', @{$removed_keywords{$keyword}};
+      @list = ((map $_.',', @list[0..$#list-1]), $list[-1]) if @list > 2;
+      splice(@list, -1, 0, 'or') if @list > 1;
+      $message .= ': this should be rewritten as '.join(' ', @list);
+    }
+    carp $message;
+  }
+
   $state->{annotations} = $orig_annotations;
 
   if ($valid) {
@@ -394,23 +424,6 @@ sub _eval {
   return $valid;
 }
 
-sub keywords { qw(definitions dependencies) }
-
-sub _eval_keyword_definitions {
-  my ($self, $data, $schema, $state) = @_;
-  carp 'no-longer-supported "definitions" keyword present (at '
-    .canonical_schema_uri($state).'): this should be rewritten as "$defs"';
-  return 1;
-}
-
-sub _eval_keyword_dependencies {
-  my ($self, $data, $schema, $state) = @_;
-  carp 'no-longer-supported "dependencies" keyword present (at'
-    .canonical_schema_uri($state)
-    .'): this should be rewritten as "dependentSchemas" or "dependentRequired"';
-  return 1;
-}
-
 has _resource_index => (
   is => 'bare',
   isa => HashRef[Dict[
@@ -528,12 +541,20 @@ sub _fetch_schema_from_uri {
     my $base = $uri->clone->fragment(undef);
     if (my $resource = $self->_get_or_load_resource($base)) {
       $subschema = $resource->{document}->get($document_path = $resource->{path}.($fragment//''));
-      undef $fragment if not length $fragment;
-      $canonical_uri = $resource->{canonical_uri}->clone->fragment($fragment);
       $document = $resource->{document};
+      my $closest_resource = first { !length($_->[1]{path})       # document root
+          || length($document_path)
+            && path($_->[1]{path})->subsumes($document_path) }    # path is above present location
+        sort { length($b->[1]{path}) <=> length($a->[1]{path}) }  # sort by length, descending
+        grep { not length Mojo::URL->new($_->[0])->fragment }     # omit anchors
+        $document->resource_pairs;
+
+      $canonical_uri = $closest_resource->[1]{canonical_uri}->clone
+        ->fragment(substr($document_path, length($closest_resource->[1]{path})));
+      $canonical_uri->fragment(undef) if not length($canonical_uri->fragment);
     }
   }
-  else {
+  else {  # we are following a URI with a plain-name fragment
     if (my $resource = $self->_get_resource($uri)) {
       $subschema = $resource->{document}->get($document_path = $resource->{path});
       $canonical_uri = $resource->{canonical_uri}->clone; # this is *not* the anchor-containing URI
@@ -567,7 +588,7 @@ JSON::Schema::Draft201909 - Validate data against a schema
 
 =head1 VERSION
 
-version 0.027
+version 0.028
 
 =head1 SYNOPSIS
 
@@ -714,7 +735,7 @@ The result is a L<JSON::Schema::Draft201909::Result> object, which can also be u
 =head2 traverse
 
   $result = $js->traverse($schema_data);
-  $result = $js->traverse($schema_data, { canonical_schema_uri => 'http://example.com' });
+  $result = $js->traverse($schema_data, { initial_schema_uri => 'http://example.com' });
 
 Traverses the provided schema data without evaluating it against any instance data. Returns the
 internal state object accumulated during the traversal, including any identifiers found therein, and
@@ -965,7 +986,7 @@ L<Understanding JSON Schema|https://json-schema.org/understanding-json-schema>:
 
 Bugs may be submitted through L<https://github.com/karenetheridge/JSON-Schema-Draft201909/issues>.
 
-I am also usually active on irc, as 'ether' at C<irc.perl.org> and C<irc.freenode.org>.
+I am also usually active on irc, as 'ether' at C<irc.perl.org> and C<irc.libera.chat>.
 
 =head1 AUTHOR
 
