@@ -4,7 +4,7 @@ package JSON::Schema::Draft201909::Document;
 # vim: set ts=8 sts=2 sw=2 tw=100 et :
 # ABSTRACT: One JSON Schema document
 
-our $VERSION = '0.027';
+our $VERSION = '0.028';
 
 use 5.016;
 no if "$]" >= 5.031009, feature => 'indirect';
@@ -50,6 +50,7 @@ has resource_index => (
     _add_resources => 'set',
     _get_resource => 'get',
     _remove_resource => 'delete',
+    _canonical_resources => 'values',
   },
   init_arg => undef,
   lazy => 1,
@@ -143,10 +144,10 @@ sub BUILD {
 
   my $original_uri = $self->canonical_uri->clone;
   my $state = $args->{_evaluator}->traverse($self->schema,
-    { canonical_schema_uri => $self->canonical_uri->clone });
+    { initial_schema_uri => $self->canonical_uri->clone });
 
   # if the schema identified a canonical uri for itself, it overrides the initial value
-  $self->_set_canonical_uri($state->{canonical_schema_uri});
+  $self->_set_canonical_uri($state->{initial_schema_uri});
 
   if (@{$state->{errors}}) {
     $self->_set_errors($state->{errors});
@@ -180,7 +181,7 @@ JSON::Schema::Draft201909::Document - One JSON Schema document
 
 =head1 VERSION
 
-version 0.027
+version 0.028
 
 =head1 SYNOPSIS
 
@@ -270,7 +271,7 @@ Returns a data structure suitable for serialization. See L</schema>.
 
 Bugs may be submitted through L<https://github.com/karenetheridge/JSON-Schema-Draft201909/issues>.
 
-I am also usually active on irc, as 'ether' at C<irc.perl.org> and C<irc.freenode.org>.
+I am also usually active on irc, as 'ether' at C<irc.perl.org> and C<irc.libera.chat>.
 
 =head1 AUTHOR
 
