@@ -4,7 +4,7 @@ package JSON::Schema::Draft201909::Utilities;
 # vim: set ts=8 sts=2 sw=2 tw=100 et :
 # ABSTRACT: Internal utilities for JSON::Schema::Draft201909
 
-our $VERSION = '0.027';
+our $VERSION = '0.028';
 
 use 5.016;
 no if "$]" >= 5.031009, feature => 'indirect';
@@ -120,7 +120,7 @@ sub is_equal {
   if ($types[0] eq 'object') {
     return 0 if keys %$x != keys %$y;
     return 0 if not is_equal([ sort keys %$x ], [ sort keys %$y ]);
-    foreach my $property (keys %$x) {
+    foreach my $property (sort keys %$x) {
       $state->{path} = jsonp($path, $property);
       return 0 if not is_equal($x->{$property}, $y->{$property}, $state);
     }
@@ -170,7 +170,7 @@ sub local_annotations {
 sub canonical_schema_uri {
   my ($state, @extra_path) = @_;
 
-  my $uri = $state->{canonical_schema_uri}->clone;
+  my $uri = $state->{initial_schema_uri}->clone;
   $uri->fragment(($uri->fragment//'').jsonp($state->{schema_path}, @extra_path));
   $uri->fragment(undef) if not length($uri->fragment);
   $uri;
@@ -260,7 +260,7 @@ sub assert_uri_reference {
       or $ref =~ /[^[:ascii:]]/
       or $ref =~ /#/
         and $ref !~ m{#$}                          # empty fragment
-        and $ref !~ m{#[A-Za-z][-A-Za-z0-9.:_]*$}  # plain-name fragment
+        and $ref !~ m{#[A-Za-z][A-Za-z0-9_:.-]*$}  # plain-name fragment
         and $ref !~ m{#/(?:[^~]|~[01])*$};         # json pointer fragment
 
   return 1;
@@ -279,7 +279,7 @@ sub assert_uri {
       or not $uri->is_abs
       or $string =~ /#/
         and $string !~ m{#$}                          # empty fragment
-        and $string !~ m{#[A-Za-z][-A-Za-z0-9.:_]*$}  # plain-name fragment
+        and $string !~ m{#[A-Za-z][A-Za-z0-9_:.-]*$}  # plain-name fragment
         and $string !~ m{#/(?:[^~]|~[01])*$};         # json pointer fragment
 
   return 1;
@@ -306,7 +306,7 @@ JSON::Schema::Draft201909::Utilities - Internal utilities for JSON::Schema::Draf
 
 =head1 VERSION
 
-version 0.027
+version 0.028
 
 =head1 SYNOPSIS
 
@@ -324,7 +324,7 @@ annotate_self
 
 Bugs may be submitted through L<https://github.com/karenetheridge/JSON-Schema-Draft201909/issues>.
 
-I am also usually active on irc, as 'ether' at C<irc.perl.org> and C<irc.freenode.org>.
+I am also usually active on irc, as 'ether' at C<irc.perl.org> and C<irc.libera.chat>.
 
 =head1 AUTHOR
 
