@@ -4,7 +4,7 @@ package JSON::Schema::Draft201909::Vocabulary;
 # vim: set ts=8 sts=2 sw=2 tw=100 et :
 # ABSTRACT: Base role for JSON Schema vocabulary classes
 
-our $VERSION = '0.027';
+our $VERSION = '0.028';
 
 use 5.016;
 no if "$]" >= 5.031009, feature => 'indirect';
@@ -22,7 +22,7 @@ sub traverse {
   $state->{evaluator}->_traverse($schema, $state);
 }
 
-sub traverse_schema {
+sub traverse_subschema {
   my ($self, $schema, $state) = @_;
 
   $state->{evaluator}->_traverse($schema->{$state->{keyword}},
@@ -52,6 +52,15 @@ sub traverse_object_schemas {
   }
 }
 
+sub traverse_property_schema {
+  my ($self, $schema, $state, $property) = @_;
+
+  return if not assert_keyword_type($state, $schema, 'object');
+
+  $state->{evaluator}->_traverse($schema->{$state->{keyword}}{$property},
+    +{ %$state, schema_path => jsonp($state->{schema_path}, $state->{keyword}, $property) });
+}
+
 sub eval {
   my ($self, $data, $schema, $state) = @_;
   $state->{evaluator}->_eval($data, $schema, $state);
@@ -71,7 +80,7 @@ JSON::Schema::Draft201909::Vocabulary - Base role for JSON Schema vocabulary cla
 
 =head1 VERSION
 
-version 0.027
+version 0.028
 
 =head1 SYNOPSIS
 
@@ -105,7 +114,7 @@ The list of keywords defined by the vocabulary. Must be implemented by the compo
 
 Traverses a subschema.
 
-=head2 traverse_schema
+=head2 traverse_subschema
 
 Recursively traverses the schema at the current keyword.
 
@@ -117,6 +126,10 @@ Recursively traverses the list of subschemas at the current keyword.
 
 Recursively traverses the (subschema) values of the object at the current keyword.
 
+=head2 traverse_property_schema
+
+Recursively traverses the subschema under one property of the object at the current keyword.
+
 =head2 eval
 
 Evaluates a subschema.
@@ -125,7 +138,7 @@ Evaluates a subschema.
 
 Bugs may be submitted through L<https://github.com/karenetheridge/JSON-Schema-Draft201909/issues>.
 
-I am also usually active on irc, as 'ether' at C<irc.perl.org> and C<irc.freenode.org>.
+I am also usually active on irc, as 'ether' at C<irc.perl.org> and C<irc.libera.chat>.
 
 =head1 AUTHOR
 
