@@ -4,7 +4,7 @@ package JSON::Schema::Draft201909::Vocabulary::Applicator;
 # vim: set ts=8 sts=2 sw=2 tw=100 et :
 # ABSTRACT: Implementation of the JSON Schema Draft 2019-09 Applicator vocabulary
 
-our $VERSION = '0.027';
+our $VERSION = '0.028';
 
 use 5.016;
 no if "$]" >= 5.031009, feature => 'indirect';
@@ -29,10 +29,12 @@ sub vocabulary { 'https://json-schema.org/draft/2019-09/vocab/applicator' }
 # - properties and patternProperties must be evaluated before additionalProperties
 # - in-place applicators and properties, patternProperties, additionalProperties must be evaluated
 #   before unevaluatedProperties
+# - contains must be evaluated before maxContains, minContains (in the Validator vocabulary)
 sub keywords {
   qw(allOf anyOf oneOf not if then else dependentSchemas
-    items additionalItems unevaluatedItems contains
-    properties patternProperties additionalProperties unevaluatedProperties propertyNames);
+    items additionalItems contains
+    properties patternProperties additionalProperties propertyNames
+    unevaluatedItems unevaluatedProperties);
 }
 
 sub _traverse_keyword_allOf { shift->traverse_array_schemas(@_) }
@@ -114,7 +116,7 @@ sub _eval_keyword_oneOf {
   }
 }
 
-sub _traverse_keyword_not { shift->traverse_schema(@_) }
+sub _traverse_keyword_not { shift->traverse_subschema(@_) }
 
 sub _eval_keyword_not {
   my ($self, $data, $schema, $state) = @_;
@@ -127,9 +129,9 @@ sub _eval_keyword_not {
   return E($state, 'subschema is valid');
 }
 
-sub _traverse_keyword_if { shift->traverse_schema(@_) }
-sub _traverse_keyword_then { shift->traverse_schema(@_) }
-sub _traverse_keyword_else { shift->traverse_schema(@_) }
+sub _traverse_keyword_if { shift->traverse_subschema(@_) }
+sub _traverse_keyword_then { shift->traverse_subschema(@_) }
+sub _traverse_keyword_else { shift->traverse_subschema(@_) }
 
 sub _eval_keyword_if {
   my ($self, $data, $schema, $state) = @_;
@@ -183,7 +185,7 @@ sub _traverse_keyword_items {
   my ($self, $schema, $state) = @_;
 
   return $self->traverse_array_schemas($schema, $state) if is_plain_arrayref($schema->{items});
-  $self->traverse_schema($schema, $state);
+  $self->traverse_subschema($schema, $state);
 }
 
 sub _eval_keyword_items {
@@ -195,7 +197,7 @@ sub _eval_keyword_items {
   goto \&_eval_keyword__items_schema;
 }
 
-sub _traverse_keyword_additionalItems { shift->traverse_schema(@_) }
+sub _traverse_keyword_additionalItems { shift->traverse_subschema(@_) }
 
 sub _eval_keyword_additionalItems {
   my ($self, $data, $schema, $state) = @_;
@@ -284,7 +286,7 @@ sub _eval_keyword__items_schema {
 sub _traverse_keyword_unevaluatedItems {
   my ($self, $schema, $state) = @_;
 
-  $self->traverse_schema($schema, $state);
+  $self->traverse_subschema($schema, $state);
 
   # remember that annotations need to be collected in order to evaluate this keyword
   $state->{configs}{collect_annotations} = 1;
@@ -344,7 +346,7 @@ sub _eval_keyword_unevaluatedItems {
   return A($state, true);
 }
 
-sub _traverse_keyword_contains { shift->traverse_schema(@_) }
+sub _traverse_keyword_contains { shift->traverse_subschema(@_) }
 
 sub _eval_keyword_contains {
   my ($self, $data, $schema, $state) = @_;
@@ -429,9 +431,7 @@ sub _traverse_keyword_patternProperties {
 
   foreach my $property (sort keys %{$schema->{patternProperties}}) {
     return if not assert_pattern({ %$state, _schema_path_suffix => $property }, $property);
-
-    $self->traverse($schema->{patternProperties}{$property},
-      +{ %$state, schema_path => jsonp($state->{schema_path}, 'patternProperties', $property) });
+    $self->traverse_property_schema($schema, $state, $property);
   }
 }
 
@@ -476,7 +476,7 @@ sub _eval_keyword_patternProperties {
   return A($state, [ uniqstr @valid_properties ]);
 }
 
-sub _traverse_keyword_additionalProperties { shift->traverse_schema(@_) }
+sub _traverse_keyword_additionalProperties { shift->traverse_subschema(@_) }
 
 sub _eval_keyword_additionalProperties {
   my ($self, $data, $schema, $state) = @_;
@@ -524,7 +524,7 @@ sub _eval_keyword_additionalProperties {
 sub _traverse_keyword_unevaluatedProperties {
   my ($self, $schema, $state) = @_;
 
-  $self->traverse_schema($schema, $state);
+  $self->traverse_subschema($schema, $state);
 
   # remember that annotations need to be collected in order to evaluate this keyword
   $state->{configs}{collect_annotations} = 1;
@@ -583,7 +583,7 @@ sub _eval_keyword_unevaluatedProperties {
   return A($state, \@valid_properties);
 }
 
-sub _traverse_keyword_propertyNames { shift->traverse_schema(@_) }
+sub _traverse_keyword_propertyNames { shift->traverse_subschema(@_) }
 
 sub _eval_keyword_propertyNames {
   my ($self, $data, $schema, $state) = @_;
@@ -626,7 +626,7 @@ JSON::Schema::Draft201909::Vocabulary::Applicator - Implementation of the JSON S
 
 =head1 VERSION
 
-version 0.027
+version 0.028
 
 =head1 DESCRIPTION
 
@@ -642,7 +642,7 @@ L<https://json-schema.org/draft/2019-09/json-schema-core.html#rfc.section.9>.
 
 Bugs may be submitted through L<https://github.com/karenetheridge/JSON-Schema-Draft201909/issues>.
 
-I am also usually active on irc, as 'ether' at C<irc.perl.org> and C<irc.freenode.org>.
+I am also usually active on irc, as 'ether' at C<irc.perl.org> and C<irc.libera.chat>.
 
 =head1 AUTHOR
 
