@@ -4,7 +4,7 @@ package JSON::Schema::Draft201909::Vocabulary::Content;
 # vim: set ts=8 sts=2 sw=2 tw=100 et :
 # ABSTRACT: Implementation of the JSON Schema Draft 2019-09 Content vocabulary
 
-our $VERSION = '0.027';
+our $VERSION = '0.028';
 
 use 5.016;
 no if "$]" >= 5.031009, feature => 'indirect';
@@ -47,7 +47,7 @@ sub _traverse_keyword_contentSchema {
 
   # since contentSchema should never be evaluated in the context of the containing schema, it is
   # not appropriate to gather identifiers found therein -- but we can still validate the subschema.
-  $self->traverse_schema($schema, +{ %$state, identifiers => [] });
+  $self->traverse_subschema($schema, +{ %$state, identifiers => [] });
 }
 
 sub _eval_keyword_contentSchema {
@@ -73,7 +73,7 @@ JSON::Schema::Draft201909::Vocabulary::Content - Implementation of the JSON Sche
 
 =head1 VERSION
 
-version 0.027
+version 0.028
 
 =head1 DESCRIPTION
 
@@ -89,7 +89,7 @@ L<https://json-schema.org/draft/2019-09/json-schema-validation.html>.
 
 Bugs may be submitted through L<https://github.com/karenetheridge/JSON-Schema-Draft201909/issues>.
 
-I am also usually active on irc, as 'ether' at C<irc.perl.org> and C<irc.freenode.org>.
+I am also usually active on irc, as 'ether' at C<irc.perl.org> and C<irc.libera.chat>.
 
 =head1 AUTHOR
 
