@@ -4,7 +4,7 @@ package JSON::Schema::Draft201909::Vocabulary::Core;
 # vim: set ts=8 sts=2 sw=2 tw=100 et :
 # ABSTRACT: Implementation of the JSON Schema Draft 2019-09 Core vocabulary
 
-our $VERSION = '0.027';
+our $VERSION = '0.028';
 
 use 5.016;
 no if "$]" >= 5.031009, feature => 'indirect';
@@ -34,19 +34,19 @@ sub _traverse_keyword_id {
   return if not assert_keyword_type($state, $schema, 'string');
 
   my $uri = Mojo::URL->new($schema->{'$id'});
+  return E($state, '$id value should not equal "%s"', $uri) if $uri eq '' or $uri eq '#';
   return E($state, '$id value "%s" cannot have a non-empty fragment', $schema->{'$id'})
     if length $uri->fragment;
 
   $uri->fragment(undef);
-  $state->{canonical_schema_uri} = $uri->is_abs ? $uri
-    : $uri->to_abs($state->{canonical_schema_uri});
+  $state->{initial_schema_uri} = $uri->is_abs ? $uri : $uri->to_abs($state->{initial_schema_uri});
   $state->{traversed_schema_path} = $state->{traversed_schema_path}.$state->{schema_path};
   $state->{schema_path} = '';
 
   push @{$state->{identifiers}},
-    $state->{canonical_schema_uri} => {
+    $state->{initial_schema_uri} => {
       path => $state->{traversed_schema_path},
-      canonical_uri => $state->{canonical_schema_uri}->clone,
+      canonical_uri => $state->{initial_schema_uri}->clone,
     };
 }
 
@@ -54,7 +54,7 @@ sub _eval_keyword_id {
   my ($self, $data, $schema, $state) = @_;
 
   if (my $canonical_uri = $state->{document}->path_to_canonical_uri($state->{document_path}.$state->{schema_path})) {
-    $state->{canonical_schema_uri} = $canonical_uri->clone;
+    $state->{initial_schema_uri} = $canonical_uri->clone;
     $state->{traversed_schema_path} = $state->{traversed_schema_path}.$state->{schema_path};
     $state->{document_path} = $state->{document_path}.$state->{schema_path};
     $state->{schema_path} = '';
@@ -91,7 +91,7 @@ sub _traverse_keyword_anchor {
 
   return if not assert_keyword_type($state, $schema, 'string');
   return E($state, '$anchor value "%s" does not match required syntax', $schema->{'$anchor'})
-    if $schema->{'$anchor'} !~ /^[A-Za-z][A-Za-z0-9_:.-]+$/;
+    if $schema->{'$anchor'} !~ /^[A-Za-z][A-Za-z0-9_:.-]*$/;
 
   my $canonical_uri = canonical_schema_uri($state);
 
@@ -103,7 +103,7 @@ sub _traverse_keyword_anchor {
 }
 
 # we already indexed the $anchor uri, so there is nothing more to do at evaluation time.
-# we explicitly do NOT set $state->{canonical_schema_uri}.
+# we explicitly do NOT set $state->{initial_schema_uri}.
 
 sub _traverse_keyword_recursiveAnchor {
   my ($self, $schema, $state) = @_;
@@ -138,7 +138,7 @@ sub _traverse_keyword_ref {
 sub _eval_keyword_ref {
   my ($self, $data, $schema, $state) = @_;
 
-  my $uri = Mojo::URL->new($schema->{'$ref'})->to_abs($state->{canonical_schema_uri});
+  my $uri = Mojo::URL->new($schema->{'$ref'})->to_abs($state->{initial_schema_uri});
   my ($subschema, $canonical_uri, $document, $document_path) = $state->{evaluator}->_fetch_schema_from_uri($uri);
   abort($state, 'EXCEPTION: unable to find resource %s', $uri) if not defined $subschema;
 
@@ -147,7 +147,7 @@ sub _eval_keyword_ref {
       %{$document->evaluation_configs},
       %$state,
       traversed_schema_path => $state->{traversed_schema_path}.$state->{schema_path}.'/$ref',
-      canonical_schema_uri => $canonical_uri, # note: maybe not canonical yet until $id is processed
+      initial_schema_uri => $canonical_uri,
       document => $document,
       document_path => $document_path,
       schema_path => '',
@@ -163,13 +163,13 @@ sub _traverse_keyword_recursiveRef {
 sub _eval_keyword_recursiveRef {
   my ($self, $data, $schema, $state) = @_;
 
-  my $target_uri = Mojo::URL->new($schema->{'$recursiveRef'})->to_abs($state->{canonical_schema_uri});
-  my ($subschema, $canonical_uri, $document, $document_path) = $state->{evaluator}->_fetch_schema_from_uri($target_uri);
-  abort($state, 'EXCEPTION: unable to find resource %s', $target_uri) if not defined $subschema;
+  my $uri = Mojo::URL->new($schema->{'$recursiveRef'})->to_abs($state->{initial_schema_uri});
+  my ($subschema, $canonical_uri, $document, $document_path) = $state->{evaluator}->_fetch_schema_from_uri($uri);
+  abort($state, 'EXCEPTION: unable to find resource %s', $uri) if not defined $subschema;
 
   if (is_type('boolean', $subschema->{'$recursiveAnchor'}) and $subschema->{'$recursiveAnchor'}) {
-    my $uri = Mojo::URL->new($schema->{'$recursiveRef'})
-      ->to_abs($state->{recursive_anchor_uri} // $state->{canonical_schema_uri});
+    $uri = Mojo::URL->new($schema->{'$recursiveRef'})
+      ->to_abs($state->{recursive_anchor_uri} // $state->{initial_schema_uri});
     ($subschema, $canonical_uri, $document, $document_path) = $state->{evaluator}->_fetch_schema_from_uri($uri);
     abort($state, 'EXCEPTION: unable to find resource %s', $uri) if not defined $subschema;
   }
@@ -179,7 +179,7 @@ sub _eval_keyword_recursiveRef {
       %{$document->evaluation_configs},
       %$state,
       traversed_schema_path => $state->{traversed_schema_path}.$state->{schema_path}.'/$recursiveRef',
-      canonical_schema_uri => $canonical_uri, # note: maybe not canonical yet until $id is processed
+      initial_schema_uri => $canonical_uri,
       document => $document,
       document_path => $document_path,
       schema_path => '',
@@ -235,7 +235,7 @@ JSON::Schema::Draft201909::Vocabulary::Core - Implementation of the JSON Schema
 
 =head1 VERSION
 
-version 0.027
+version 0.028
 
 =head1 DESCRIPTION
 
@@ -251,7 +251,7 @@ L<https://json-schema.org/draft/2019-09/json-schema-core.html#rfc.section.8>.
 
 Bugs may be submitted through L<https://github.com/karenetheridge/JSON-Schema-Draft201909/issues>.
 
-I am also usually active on irc, as 'ether' at C<irc.perl.org> and C<irc.freenode.org>.
+I am also usually active on irc, as 'ether' at C<irc.perl.org> and C<irc.libera.chat>.
 
 =head1 AUTHOR
 
