@@ -4,7 +4,7 @@ package JSON::Schema::Draft201909::Vocabulary::Validation;
 # vim: set ts=8 sts=2 sw=2 tw=100 et :
 # ABSTRACT: Implementation of the JSON Schema Draft 2019-09 Validation vocabulary
 
-our $VERSION = '0.027';
+our $VERSION = '0.028';
 
 use 5.016;
 no if "$]" >= 5.031009, feature => 'indirect';
@@ -288,7 +288,7 @@ sub _traverse_keyword_dependentRequired {
 
   return if not assert_keyword_type($state, $schema, 'object');
 
-  foreach my $property (keys %{$schema->{dependentRequired}}) {
+  foreach my $property (sort keys %{$schema->{dependentRequired}}) {
     E({ %$state, _schema_path_suffix => $property }, 'dependentRequired value is not an array'), next
       if not is_type('array', $schema->{dependentRequired}{$property});
 
@@ -308,7 +308,7 @@ sub _eval_keyword_dependentRequired {
   return 1 if not is_type('object', $data);
 
   my $valid = 1;
-  foreach my $property (keys %{$schema->{dependentRequired}}) {
+  foreach my $property (sort keys %{$schema->{dependentRequired}}) {
     next if not exists $data->{$property};
 
     if (my @missing = grep !exists($data->{$_}), @{$schema->{dependentRequired}{$property}}) {
@@ -347,7 +347,7 @@ JSON::Schema::Draft201909::Vocabulary::Validation - Implementation of the JSON S
 
 =head1 VERSION
 
-version 0.027
+version 0.028
 
 =head1 DESCRIPTION
 
@@ -363,7 +363,7 @@ L<https://json-schema.org/draft/2019-09/json-schema-validation.html#rfc.section.
 
 Bugs may be submitted through L<https://github.com/karenetheridge/JSON-Schema-Draft201909/issues>.
 
-I am also usually active on irc, as 'ether' at C<irc.perl.org> and C<irc.freenode.org>.
+I am also usually active on irc, as 'ether' at C<irc.perl.org> and C<irc.libera.chat>.
 
 =head1 AUTHOR
 
